# python script to produce graphs of co2, temperature over 300000 years, and compute lag

import os.path
import urllib
from hashlib import sha256
import numpy as np
import matplotlib.pyplot as plt

# sources
url1 = 'https://data.giss.nasa.gov/gistemp/graphs/graph_data/Global_Mean_Estimates_based_on_Land_and_Ocean_Data/graph.txt'
url2 = 'https://data.giss.nasa.gov/gistemp/graphs/graph_data/Temperature_Anomalies_over_Land_and_over_Ocean/graph.txt'
url3 = 'https://www.nsstc.uah.edu/data/msu/v6.0/tlt/uahncdc_lt_6.0.txt'

# get sources if necessary
if not os.path.isfile('nasa_surface_air.txt'):
    file1 = urllib.urlretrieve(url1, 'nasa_surface_air.txt')
with open('nasa_surface_air.txt') as f:
    content1 = f.read()
hash1 = sha256(content1).hexdigest()
if hash1 != '338cd026bd6065764f9f13ebb77bddafb7b5e5d3828ce279e0551483280757e4':
    print("error! file hashes do not match:", url1)
    print("SHA2-256 hash should be: 338cd026bd6065764f9f13ebb77bddafb7b5e5d3828ce279e0551483280757e4")
    print("but is: ", hash1)

if not os.path.isfile('nasa_surface_sea.txt'):
    file2 = urllib.urlretrieve(url2, 'nasa_surface_sea.txt')
with open('nasa_surface_sea.txt') as f:
    content2 = f.read()
hash2 = sha256(content2).hexdigest()
if hash2 != '08ee176487708311cec14b0ebccbb06d3fe6fd07cb631d7b8bbafb48a150fb45':
    print("error! file hashes do not match:", url2)
    print("SHA2-256 hash should be: 08ee176487708311cec14b0ebccbb06d3fe6fd07cb631d7b8bbafb48a150fb45")
    print("but is: ", hash2)

if not os.path.isfile('uah_satellite.txt'):
    file3 = urllib.urlretrieve(url3, 'uah_satellite.txt')
with open('uah_satellite.txt') as f:
    content3 = f.read()
hash3 = sha256(content3).hexdigest()
if hash3 != 'fb070fcf31a020b8f2fc74c998b0be584973f38205bd2631324c4aedde906493':
    print("error! file hashes do not match:", url3)
    print("SHA2-256 hash should be: fb070fcf31a020b8f2fc74c998b0be584973f38205bd2631324c4aedde906493")
    print("but is: ", hash3)
    
# parse files
d1 = dict()
linectr = 0
for line in content1.split('\n'):
    linectr += 1
    if (len(line) != 0 and line[0] == '#') or linectr < 6:
        continue
    words = line.split(' ')
    wds = [w for w in words if w != '']
    if len(wds) == 0:
        continue
    d1[int(wds[0])] = float(wds[1])

d2 = dict()
linectr = 0
for line in content2.split('\n'):
    linectr += 1
    if (len(line) != 0 and line[0] == '#') or linectr < 6:
        continue
    words = line.split(' ')
    wds = [w for w in words if w != '']
    if len(wds) == 0:
        continue
    d2[int(wds[0])] = float(wds[4])

d3 = dict()
linectr = 0
for line in content3.split('\n'):
    linectr += 1
    if (len(line) != 0 and line[0] == '#') or linectr <= 1 or linectr >= 472:
        continue
    words = line.split(' ')
    wds = [w for w in words if w != '']
    if len(wds) == 0:
        continue
    d3[float(wds[0]) + (float(wds[1])-1)/12] = float(wds[2])

# to numpy
data1 = np.array(list(sorted(d1.items())))
data2 = np.array(list(sorted(d2.items())))
data3 = np.array(list(sorted(d3.items())))

# Sat data has mean 0 over time span 1980-2020, but other data has
# mean 0 over time span 1880-2020. It is okay to shift sat data so
# its mean overlaps with the mean of other data for 1980-2020.
mean1 = np.mean(data1[100:, 1])
mean2 = np.mean(data2[100:, 1])
data3 += (mean1 + mean2)/2

# plot
handle_sat = plt.plot(data3[...,0], data3[...,1], '-g', linewidth=1.0, label='satellite')
handle_air = plt.plot(data1[...,0], data1[...,1], '-r', linewidth=2.0, label='surface air')
handle_sea = plt.plot(data2[...,0], data2[...,1], '-b', linewidth=2.0, label='surface sea')
plt.xlabel('year')
#plt.ylabel('temperature anomaly $(^\circ C)$')
plt.title('temperature anomaly ($^\circ C$) in past 140 years')
plt.grid(True)
ax = plt.gca()
plt.legend(loc=2)
#plt.show()
plt.savefig('temp_anomaly.svg')
#plt.savefig('temp_anomaly.png')


